/*	EXAMPLE PROGRAM WITH INTERRUPT HANDLING
 *	Copyright (c) 1996 by COSMIC Software 
 *
 *	Each character received is copied in a buffer
 *	by the interrupt routine. The main program reads
 *	characters from the buffer and echoes them.
 */

@dir volatile char SC1BDL @0xc9;	/* SCI 1 baud rate low */
@dir volatile char SC1CR2 @0xcb;	/* SCI 1 control register 2 */
@dir volatile char SC1SR1 @0xcc;	/* SCI 1 status register 1 */
@dir volatile char SC1DRH @0xce;	/* SCI 1 data register high */
@dir volatile char SC1DRL @0xcf;	/* SCI 1 data register low */

#define SIZE	512	/* buffer size */
#define TDRE	0x80	/* transmit ready bit */

/*	Authorize interrupts.
 */
#define cli()	_asm("andcc #$EF\n")

/*	Some variables.
 */
char buffer[SIZE];	/* reception buffer */
char *ptlec;		/* read pointer */
char *ptecr;		/* write pointer */

/*	Character reception.
 *	Loops until a character is received.
 */
char getch(void)
	{
	char c;				/* character to be returned */

	while (ptlec == ptecr)		/* equal pointers => loop */
		;
	c = *ptlec++;			/* get the received char */
	if (ptlec >= &buffer[SIZE])	/* put in in buffer */
		ptlec = buffer;
	return (c);
	}

/*	Send a char to the SCI 1.
 */
void outch(char c)
	{
	while (!(SC1SR1 & TDRE))	/* wait for READY */
		;
	SC1DRL = c;			/* send it */
	}

/*	Character reception routine.
 *	This routine is called on interrupt.
 *	It puts the received char in the buffer.
 */
@interrupt void recept(void)
	{
	SC1SR1;				/* clear interrupt */
	*ptecr++ = SC1DRL;		/* get the char */
	if (ptecr >= &buffer[SIZE])	/* put it in buffer */
		ptecr = buffer;
	}

/*	Main program.
 *	Sets up the SCI and starts an infinite
 *	loop of receive transmit.
 */
void main(void)
	{
	ptecr = ptlec = buffer;		/* initialize pointers */
	SC1BDL = 55;			/* initialize SCI */
	SC1CR2 = 0x2c;			/* parameters for interrupt */
	cli();				/* authorize interrupts */
	for (;;)			/* loop */
		outch(getch());		/* get and put a char */
	}
